Package: sebkc
Type: Package
Title: Surface Energy Balance and Crop Coefficient Evapotranspiration
        Estimation
Version: 1.0-6
Date: 2026-07-15
Authors@R: person(given = "George", family = "Owusu",
    email = "owusugeorge@ug.edu.gh", role = c("aut", "cre"))
Description: Computes and integrates surface energy balance components of
    evapotranspiration (ET), sensible heat (H), soil heat flux (G) and net
    radiation (Rn) into the Food and Agriculture Organization (FAO)
    Irrigation and Drainage Paper 56 (FAO-56) water balance model. The
    package can perform single crop coefficient (Kc), dual Kc and the
    integration of thermal-based evaporative fractions in a water balance
    model. The surface energy balance models include Two-Source Surface
    Energy Balance (TSEB) models, namely the Priestley-Taylor TSEB
    (TSEB-PT), the Penman-Monteith TSEB (TSEB-PM) and TSEB-Parallel, as
    well as One-Source Surface Energy Balance (OSEB) models, namely the
    Surface Energy Balance Algorithm for Land (SEBAL), Mapping
    Evapotranspiration at high Resolution with Internalized Calibration
    (METRIC), Surface Energy Balance Index (SEBI), Simplified Surface
    Energy Balance (SSEB) and Surface Energy Balance System (SEBS). Methods
    are described in Allen et al. (2007)
    <doi:10.1061/(ASCE)0733-9437(2007)133:4(380)> and Bastiaanssen et al.
    (1998) <doi:10.1016/S0022-1694(98)00253-4>.
Depends: R (>= 3.5.0), raster (>= 2.5), sp (>= 1.2)
Imports: gstat, stats, graphics, grDevices, utils
License: GPL-2
Encoding: UTF-8
URL: https://github.com/gowusu/sebkc
BugReports: https://github.com/gowusu/sebkc/issues
NeedsCompilation: no
Config/roxygen2/version: 8.0.0
Packaged: 2026-07-15 11:34:25 UTC; PC
Author: George Owusu [aut, cre]
Maintainer: George Owusu <owusugeorge@ug.edu.gh>
Repository: CRAN
Date/Publication: 2026-07-23 12:40:08 UTC
Built: R 4.6.0; ; 2026-07-23 14:38:50 UTC; unix
