# Generated by roxygen2: do not edit by hand

S3method(ETo,default)
S3method(ETohr,default)
S3method(coldTs,default)
S3method(coldTs2,default)
S3method(coldhot,default)
S3method(hotTs,default)
S3method(hotTs2,default)
S3method(kc,default)
S3method(landsat578,default)
S3method(plot,kc)
S3method(sebal,default)
S3method(sebi,default)
S3method(sebkcstack,default)
S3method(sebs,default)
S3method(sseb,default)
S3method(ssebi,default)
S3method(tseb,default)
export(ETo)
export(ETohr)
export(biomass)
export(cal.kc)
export(coldTs)
export(coldTs2)
export(coldhot)
export(cropsebkc)
export(dl)
export(fit.kc)
export(fitting)
export(hotTs)
export(hotTs2)
export(invdist)
export(kc)
export(landsat578)
export(lonlatreproject)
export(mapmatrix)
export(rastercon)
export(sebal)
export(sebi)
export(sebkc.tryCatch)
export(sebkcstack)
export(sebs)
export(sseb)
export(ssebi)
export(tseb)
export(wdi)
export(weather)
export(writesebkc)
export(xyextract)
import(raster)
import(sp)
importFrom(grDevices,dev.new)
importFrom(graphics,abline)
importFrom(graphics,axis)
importFrom(graphics,legend)
importFrom(graphics,par)
importFrom(gstat,gstat)
importFrom(stats,coef)
importFrom(stats,cor)
importFrom(stats,kmeans)
importFrom(stats,lm)
importFrom(stats,na.omit)
importFrom(stats,pf)
importFrom(utils,data)
importFrom(utils,read.csv)
importFrom(utils,read.delim)
importFrom(utils,read.table)
importFrom(utils,write.csv)
importFrom(utils,write.table)
