#' Historical Weather data of Kumasi from 1960 to 2015
#'
#' A dataset containing Rain, maximum and minimum temperature, wind speed, sunshine hours,
#' and minimum and maximum Relative Humidity.
#' 
#' @name kumasi1960_2015
#' @docType data
#' @author George Owusu
#' @format A data frame with 20454 rows and 8 variables:
#' \describe{
#'   \item{DOY}{Day of the Year in dates}
#'   \item{Tmax}{Maximum Temperature [oC]}
#'   \item{Tmin}{Minimum Temperature [oC]}
#'   \item{RHmin}{Minimum Relative Humidity [per cent]}
#'   \item{RHmax}{Maximum Relative Humidity [per cent]}
#'   \item{n}{sunshine hours}
#'   \item{uz}{wind speed [m/s]}
#'   \item{P}{Daily precipitation [mm]}
#' }
#' 
NULL

#' A 2.6 RCP downscaled Weather data of Kumasi from 2016 to 2050
#'
#' A dataset containing Rain, maximum and minimum temperature, wind speed, sunshine hours,
#' and minimum and maximum Relative Humidity. The RCPs 2.6 assumes that green house 
#' gas will peak in between 2010-2020 and declines throughout 21st Century. 
#' 
#' @name kumasircp262016_2050
#' @docType data
#' @author George Owusu
#' @format A data frame with 12784 rows and 8 variables:
#' \describe{
#'   \item{DOY}{Day of the Year in dates}
#'   \item{Tmax}{Maximum Temperature [oC]}
#'   \item{Tmin}{Minimum Temperature [oC]}
#'   \item{RHmin}{Minimum Relative Humidity [per cent]}
#'   \item{RHmax}{Maximum Relative Humidity [per cent]}
#'   \item{n}{sunshine hours}
#'   \item{uz}{wind speed [m/s]}
#'   \item{P}{Daily precipitation [mm]}
#' }
#' 
NULL

#' A 4.5 RCP downscaled Weather data of Kumasi from 2016 to 2050
#'
#' A dataset containing Rain, maximum and minimum temperature, wind speed, sunshine hours,
#' and minimum and maximum Relative Humidity. The RCP 4.5 assumes that green house 
#' gas will peak in 2040 and declines throughout 21st Century. 
#' 
#' @name kumasircp452016_2050
#' @docType data
#' @author George Owusu
#' @format A data frame with 12784 rows and 8 variables:
#' \describe{
#'   \item{DOY}{Day of the Year in dates}
#'   \item{Tmax}{Maximum Temperature [oC]}
#'   \item{Tmin}{Minimum Temperature [oC]}
#'   \item{RHmin}{Minimum Relative Humidity [per cent]}
#'   \item{RHmax}{Maximum Relative Humidity [per cent]}
#'   \item{n}{sunshine hours}
#'   \item{uz}{wind speed [m/s]}
#'   \item{P}{Daily precipitation [mm]}
#' }
#' 
NULL

#' A 8.5 RCP downscaled Weather data of Kumasi from 2016 to 2050
#'
#' A dataset containing Rain, maximum and minimum temperature, wind speed, sunshine hours,
#' and minimum and maximum Relative Humidity. The RCP 8.5 assumes that green house 
#' gas will rise throughout 21st Century. 
#' 
#' @name kumasircp852016_2050
#' @docType data
#' @author George Owusu
#' @format A data frame with 12784 rows and 8 variables:
#' \describe{
#'   \item{DOY}{Day of the Year in dates}
#'   \item{Tmax}{Maximum Temperature [oC]}
#'   \item{Tmin}{Minimum Temperature [oC]}
#'   \item{RHmin}{Minimum Relative Humidity [per cent]}
#'   \item{RHmax}{Maximum Relative Humidity [per cent]}
#'   \item{n}{sunshine hours}
#'   \item{uz}{wind speed [m/s]}
#'   \item{P}{Daily precipitation [mm]}
#' }
#' 
NULL