% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kumasi.R
\docType{data}
\name{kumasircp262016_2050}
\alias{kumasircp262016_2050}
\title{A 2.6 RCP downscaled Weather data of Kumasi from 2016 to 2050}
\format{
A data frame with 12784 rows and 8 variables:
\describe{
  \item{DOY}{Day of the Year in dates}
  \item{Tmax}{Maximum Temperature [oC]}
  \item{Tmin}{Minimum Temperature [oC]}
  \item{RHmin}{Minimum Relative Humidity [per cent]}
  \item{RHmax}{Maximum Relative Humidity [per cent]}
  \item{n}{sunshine hours}
  \item{uz}{wind speed [m/s]}
  \item{P}{Daily precipitation [mm]}
}
}
\description{
A dataset containing Rain, maximum and minimum temperature, wind speed, sunshine hours,
and minimum and maximum Relative Humidity. The RCPs 2.6 assumes that green house 
gas will peak in between 2010-2020 and declines throughout 21st Century.
}
\author{
George Owusu
}
