% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sebkcerror.R
\name{sebkc.tryCatch}
\alias{sebkc.tryCatch}
\title{Internal function to Catch  errors and warnings}
\usage{
sebkc.tryCatch(expr)
}
\arguments{
\item{expr}{an  R expression to evaluate}
}
\value{
value a list with 'value' and 'warning', where value may be an error caught
}
\description{
Internal function to Catch  errors and warnings
}
\examples{
thiserror=str( sebkc.tryCatch( log( "a" ) ) )$value
}
