% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/xyextract.R
\name{mapmatrix}
\alias{mapmatrix}
\title{Internal function for doing interpolation}
\usage{
mapmatrix(xydata, i, map)
}
\arguments{
\item{xydata}{x and y data}

\item{i}{index}

\item{map}{geographical map}
}
\value{
map
}
\description{
Internal function for doing interpolation
}
\examples{
\dontrun{
folder=system.file("extdata","stack",package="sebkc")
sebiauto=sebi(folder=folder,welev=317,Tmax=31,Tmin=28)
points=sampleRandom(sebiauto$EF,100,sp=TRUE)
pt=cbind(points@coords,points@data)
longitude=pt$x
latitude=pt$y
value=pt$layer
mapmatrix=mapmatrix(xydata,2)
map=invdist(longitude,latitude,var=value,ext=sebiauto$EF)
}
}
\author{
George Owusu
}
