% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cropsebkc.R
\name{cropsebkc}
\alias{cropsebkc}
\title{Reprojecting and cropping images}
\usage{
cropsebkc(x, y)
}
\arguments{
\item{x}{object to be extracted}

\item{y}{extent object or raster object or polygon from which an Extent 
object can be extracted. A polygon or raster will be reprojected to 
conform x}
}
\value{
cropped  x and reprojected y
}
\description{
This function returns the subset of x object as defined by y extent object. 
See \code{\link[raster]{crop}} and \code{\link[raster]{extent}} for more details.
The difference between \code{\link[raster]{crop}}  and cropsebkc is the ability for the
latter to reproject y object with x object projection.
}
\examples{
\dontrun{
folder=system.file("extdata","stack",package="sebkc")
stack=sebkcstack(folder=folder)
data=cropsebkc(stack$data,stack$data)
}

}
\seealso{
\code{\link[raster]{crop}}
}
\author{
George Owusu
}
