% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/biomass.R
\name{invdist}
\alias{invdist}
\title{Inverse distance interpolation}
\usage{
invdist(longitude, latitude, var, ext, idp = 0.5)
}
\arguments{
\item{longitude}{The longitude of the measurement site 
i.e. geographical coordinates in decimal degrees for the weather station.
 It should be negative for West and positive for East.}

\item{latitude}{geographical coordinates in decimal degrees. It should be negative
for southern hemisphere}

\item{var}{numeric variable. The length should be same as the length of 
latitude and longitude}

\item{ext}{RasterLayer object.}

\item{idp}{Neighbourhood parameter}
}
\value{
var: interpolated values
}
\description{
This function performs inverse distance interpolation based 
on projection and geometry of input spatial data 'ext'.
}
\examples{
 \dontrun{
#Get ext data
folder=system.file("extdata","stack",package="sebkc")
data=landsat578(data=folder, welev=362)
temp=data$Ts
latitude=seq(7.544,7.590,0.001)
longitude=seq(-1.211,-1.187,0.001)
var=seq(20,30,0.2)
latitude=latitude[1:length(longitude)]
var=var[1:length(longitude)]
map=invdist(longitude,latitude,var,ext=temp)
}
}
\author{
George Owusu
}
