% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/conditional.R
\name{rastercon}
\alias{rastercon}
\title{Spatial Conditional function on raster object}
\usage{
rastercon(condition, trueValue, falseValue)
}
\arguments{
\item{condition}{logical}

\item{trueValue}{logical}

\item{falseValue}{logical}
}
\value{
Return True or False
}
\description{
It computes equivalent of logical if function in R
}
\examples{
\dontrun{
NDVI=raster(system.file("extdata","NDVI.grd",package="sebkc"))
albedo=raster(system.file("extdata","albedo.grd",package="sebkc"))
LAI=raster(system.file("extdata","LAI.grd",package="sebkc"))
  eNB=rastercon( NDVI<0 & albedo<0.47,0.99,rastercon(LAI>=3,0.98,0.97+(LAI*0.0033)))
  }
}
\author{
George Owusu
}
