% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/xyextract.R
\name{xyextract}
\alias{xyextract}
\title{Reprojecting and extracting Raster values}
\usage{
xyextract(map, longitude, latitude)
}
\arguments{
\item{map}{a raster object.}

\item{longitude}{The longitude of the measurement site 
i.e. geographical coordinates in decimal degrees for the weather station.
 It should be negative for West and positive for East.}

\item{latitude}{geographical coordinates in decimal degrees. It should be negative
for southern hemisphere}
}
\value{
data
}
\description{
The function reprojects longitudes and latitudes into map
coordinates and extract raster values
}
\examples{
\dontrun{
lonlat=read.table(system.file("extdata","sys","xyvalues.txt",package="sebkc"),header=TRUE)
longitude=lonlat$longitude
latitude=lonlat$latitude
folder=system.file("extdata","stack",package="sebkc")
data=landsat578(data=folder, welev=362)
Ts=data$Ts
Tsdata=xyextract(Ts,longitude,latitude)

}
}
\author{
George Owusu
}
