% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DL.R
\name{dl}
\alias{dl}
\title{Day length calculation}
\usage{
dl(latitude, DOY, model = "CBM", Tmax = NULL, Tmin = NULL, p = 0.5)
}
\arguments{
\item{latitude}{geographical coordinates in decimal degrees. It should be negative
for southern hemisphere}

\item{DOY}{Numeric or Date [YYYY-mm-dd]. Day of the Year. 
If you give data in the form of date [YYYY-mm-dd], it will be converted to DOY}

\item{model}{character Type of model:"CBM" (Schoolfield, 1982),"BGC" (Running & Coughlan, 1988),
"CERES" (Ritchie, 1991) or "FAO56"}

\item{Tmax}{Numeric. Maximum air Temperature in degree Celsius}

\item{Tmin}{Numeric. Minimum air Temperature in degree Celsius}

\item{p}{numeric. CMB parameter}
}
\value{
DL
}
\description{
Day length calculation
}
\examples{
DOY="2001-8-1"
latitude=0
model="CBM"
Tmax=31
Tmin=26
mod=dl(latitude,DOY,model,Tmax,Tmin=Tmin)
}
\references{
\itemize{
\item Schoolfield R. (1982). Expressing daylength as a function of latitude and Julian date.
\item Running S. W. and Coughlan J. C. (1988). A general model of forest ecosystem processes 
for regional applications I. Hydrologic balance, canopy gas exchange and primary production 
processes. Ecological Modelling, 42(2), 125-154 http://dx.doi.org/10.1016/0304-3800(88)90112-3.
\item Ritchie J. T. (1991). Wheat Phasic Development. Modeling Plant and Soil Systems. 
 Agronomy Monograph, 31.
\item Allen R. G., Pereira L. S., Raes D. and Smith M. (1998). Crop evapotranspiration: 
Guidelines for computing crop water requirements. FAO Irrigation and Drainage Paper, 56, 300.
}
}
\author{
George Owusu
}
