% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fitting.R
\name{fitting}
\alias{fitting}
\title{Goodness of fit Test}
\usage{
fitting(obs, est, P = 1)
}
\arguments{
\item{obs}{Observed data}

\item{est}{Estimated data}

\item{P}{Number of parameters}
}
\value{
\describe{
\item{r2:}{  R-square [-]}
 \item{r2_adj:}{ An adjusted R-square [-]}
\item{d:}{ Index of agreement[-]}
\item{p_value:}{ p value}
\item{sse:}{ Sum of square error}
\item{sst:}{ Sum of square total}
\item{AIC:}{ AIC}
\item{BIC:}{ BIC}
\item{RMSE:}{ Root Mean Square Error}
\item{RMSE_adj:}{ adjusted Root Mean Square Error}
\item{NRMSE}{ Normalised Root Mean Square Error}
\item{NRMSE_adj:}{ adjusted Normalised Root Mean Square Error}
\item{E:}{ Nash Sutcliffe Coefficient}
\item{b:}{ coefficients of regression}
\item{AAE:}{ average absolute error}
\item{ARE:}{ average relative error}
\item{MBE:}{ Mean Bias  error}
\item{MPE:}{ Mean Percentage error}
}
}
\description{
Goodness of fit Test
}
\examples{
file=system.file("extdata","sys","irrigation.txt",package="sebkc")
data=read.table(file,header=TRUE)  
obs=data$ETo
est=data$ETc
P=4
mod=fitting(obs,est,P)
r2=mod$r2
}
