% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sebkc-package.R
\docType{package}
\name{sebkc-package}
\alias{sebkc}
\alias{sebkc-package}
\title{sebkc: Surface Energy Balance and Crop Coefficient Evapotranspiration
Estimation}
\description{
Computes and integrates surface energy balance components of
evapotranspiration into the FAO56 water balance model, providing the
SEBAL, METRIC, SEBI, SSEB, SEBS and TSEB models.
}
\details{
Generics that the 'raster' package also provides as methods (such as plot,
hist, quantile, text, lines and points) are intentionally not imported
here, because they are made available through import(raster); importing
them again would trigger "replacing previous import" warnings.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/gowusu/sebkc}
  \item Report bugs at \url{https://github.com/gowusu/sebkc/issues}
}

}
\author{
\strong{Maintainer}: George Owusu \email{owusugeorge@ug.edu.gh}

Authors:
\itemize{
  \item George Owusu \email{owusugeorge@ug.edu.gh}
}

}
\keyword{internal}
